@@ -138,7 +138,6 @@ const (
 
 var (
 	// Errors that could happen while parsing a string.
-	//nolint:revive
 	ErrFormatWrong = errors.New("quantities must match the regular expression '" + splitREString + "'")
 	ErrNumeric     = errors.New("unable to parse numeric part of quantity")
 	ErrSuffix      = errors.New("unable to parse quantity's suffix")
@@ -258,7 +257,7 @@ Suffix:
 	// we encountered a non decimal in the Suffix loop, but the last character
 	// was not a valid exponent
 	err = ErrFormatWrong
-	// nolint:nakedret
+	//nolint:nakedret
 	return
 }
 
@@ -293,7 +292,7 @@ func ParseQuantity(str string) (Quantity, error) {
 		switch {
 		case exponent >= 0 && len(denom) == 0:
 			// only handle positive binary numbers with the fast path
-			mantissa = int64(int64(mantissa) << uint64(exponent))
+			mantissa <<= uint64(exponent)
 			// 1Mi (2^20) has ~6 digits of decimal precision, so exponent*3/10 -1 is roughly the precision
 			precision = 15 - int32(len(num)) - int32(float32(exponent)*3/10) - 1
 		default:
@@ -313,7 +312,7 @@ func ParseQuantity(str string) (Quantity, error) {
 			if err != nil {
 				return Quantity{}, ErrNumeric
 			}
-			if result, ok := int64Multiply(value, int64(mantissa)); ok {
+			if result, ok := int64Multiply(value, mantissa); ok {
 				if !positive {
 					result = -result
 				}
@@ -579,9 +578,9 @@ func (q Quantity) MarshalJSON() ([]byte, error) {
 	// if CanonicalizeBytes needed more space than our slice provided, we may need to allocate again so use
 	// append
 	result = result[:1]
-	result = append(result, number...) // nolint: makezero
-	result = append(result, suffix...) // nolint: makezero
-	result = append(result, '"')       // nolint: makezero
+	result = append(result, number...) //nolint: makezero
+	result = append(result, suffix...) //nolint: makezero
+	result = append(result, '"')       //nolint: makezero
 	return result, nil
 }
 
